#!/usr/local/bin/perl
#
# ************************************************************************************
# This file available at http://www.moreover.com/dev/cache/get_feed_to_cache.pl
#
# Email: support @ moreover . com  for support, questions etc.
#
# Version 2.07:
# =============
# In version 2.x:
# ---------------
# o Ability to use keyword searches, source-specific feeds etc, as well as the
#   standard Moreover news categories.
#   NB: this means that URLs used with any previous version of this script will
#       no longer be valid.
# o Debugging code to help with installation.
# o Potential security hole fixed.
#   This version should be used instead of any previous versions.
#
# In versions 2.01-2.04:
# ----------------------
# o Saves cached files without characters which confuse certain web servers
# o Uses ARGV instead of CGI to make inclusion in crontab files easier
# o Path to cached_newsfeeds directory included, to allow use from a cron file
# o Allows '%' signs in cron file to be replaced with '_CHAR'
# o Correct MIME type for RSS
# o Multiple k= fields supported
#
# In version 2.05:
# ----------------
# o n= field supported
#
# In version 2.06:
# ----------------
# o feed= field supported
#
# ************************************************************************************
#
# Last updated : 08.February.01
#
# ************************************************************************************
# You may copy and distribute verbatim or modified copies of the Program's source code 
# as you receive it, in any medium, provided that you conspicuously and appropriately 
# publish on each copy this copyright notice and disclaimer of warranty shown below;
# and give any other recipients of the Program a copy of this License along with the
# Program.
# 
# BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
# TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING THE 
# COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY
# OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS 
# TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE
# DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
#
# Copyright 2000 Moreover.com Inc. All rights reserved.
# *************************************************************************************
#
# This program should be called regularly (e.g. every 15 minutes) by an entry in a
# 'cron' file. It obtains the latest newsfeed from Moreover and saves a local copy.
# Referring to this local copy, instead of using the Moreover URL will improve loading
# times for your webpage, since the user will already have a connection open to your
# server. Connecting to Moreover's newsfeed servers requires a new connection from the
# user, which may result in a few second delay. This is for servers with Perl
# installed, and a directory suitable for executing Perl programs (cgi-bin,
# cgi-local etc), as well as the ability to run a 'cron' script at regular 15 minute
# intervals.
#
# Note the code also displays the requested feed so it can be called and tested from
# a browser.
#
# See http://www.moreover.com/dev/cache/ for this and other code listings, including code
# for servers that cannot run cron scripts.
#
# INSTALLATION INSTRUCTIONS:
#
# o copy this file to your CGI script directory e.g. http://www.yoursite.com/cgi-local/
#   (if you wish to test via a browser) or elsewhere if tested from command line, and
#   adjust $base_dir as appropriate
#   for example, $base_dir = "/home/apache/hosts/mysite/WWWDocs/cgi-local/news";
#   would save cached files in /home/apache/hosts/mysite/WWWDocs/cached_newsfeeds
#
# o set file permissions to 755 (if UNIX/Linux server) so that it can be run
#
# o create a directory http://www.yoursite.com/cached_newsfeeds/
#   if you choose another directory then adjust $cache_dir as appropriate
#
# o check file permissions of the caching directory to check that it can be written to by
#   the script process owner
#
# MODIFICATIONS NEEDED:
#
# o Set the first line of this listing to the location of Perl on your server
#
# o Set $cache_time to be whatever multiple of 15 minutes you require.
#
# o Set $timeout to be the maximum number of seconds the program wi	ll wait to
#   get a newsfeed from the Moreover server
#
# o Find out which Perl HTTP and LWP modules you have installed, or install them as
#   necessary. Make sure the correct modules are referenced where it says '# Perl modules'
#
# TESTING:
#
# o Try calling the program from a browser window to see if it runs:
#
#      1) use: http://www.yoursite.com/cgi-local/get_feed_to_cache.pl?c+Top%20stories+o+html
#         substituting /cgi-local/ for the name of your CGI directory
#
#      2) then look at what the feed should look like:
#         http://p.moreover.com/cgi-local/page?c=Top%20stories&o=html
#
#      3) next, see if the file http://www.yoursite.com/cached_newsfeeds/c=Top__stories.html
#         exists. This file should be the same as the results to part 2 above.
#
#      4) some webservers or filesystems have problems with filenames containing certain
#         characters. If you can identify which are problematic, edit %problem_chars
#         to replace them in the filename with something else.
#
# USAGE:
#
# o to use:
#   1) add the query-string as command line arguments, in pairs, separated by spaces.
#      e.g. perl <full path to file>/get_feed_to_cache.pl c Top%20stories k internet o js
#           will get
#           http://p.moreover.com/cgi-local/page?c=Top%20stories&k=internet&o=js
#
#   2) create a line in a cron script to run this code at 15 minute intervals
#      e.g.
#      0,15,30,45 * * * * perl <full path to file>/get_feed_to_cache.pl c Top%20stories o js
#      Note: you must find the full path to the file, and not use the URL
#
#   3) (JavaScript Wizard users)
#      change the URL of the feed to refer to your local copy
#      e.g. from
#      <SCRIPT LANGUAGE="JavaScript" SRC="http://p.moreover.com/cgi-local/page?c=Top%20stories&o=js">
#      to
#      <SCRIPT LANGUAGE="JavaScript" SRC="http://www.yoursite.com/cached_newsfeeds/c=Top__stories.js">
#
# o You can find lists of categories and newsfeed formats at http://www.moreover.com/categories/
#   The 'c=' values can be found at http://www.moreover.com/categories/category_list_c_field.html
#
# o for our old system 'login' accounts, using your email address as username, use 'u='
#   (e.g. http://www.yoursite.com/cgi-local/cached_feed.pl?u=<email_address>&o=... )
#
# o multiple k= fields are now supported. See http://www.moreover.com/dev/custom for more details
#    - use k, k2, and k3 to pass these values
#      i.e. http://p.moreover.com/cgi-local/page?k=%20bond%20|municipal&k=!james&o=xml
#      would need k=%20bond%20|municipal and k2=!james
#
# o non-edited (feed=) categories are now supported
#    i.e. http://p.moreover.com/cgi-local/page?feed=533&o=js
#    would need feed 533 o js
#
# HINTS:
#
# o Get the feed working first, before attempting to use this caching script. This means
#   trying the feed as a URL in a browser
#   (e.g. http://p.moreover.com/cgi-local/page?c=Top%20stories&o=html - setting 'o'
#   as 'html' creates a browser readable feed)
#   as well as integrating a standard Moreover feed successfully within your website
#   or application. This eradicates most problems (incorrect case of letters, etc)
#
# *************************************************************************************

# Set this to false once the program is installed and modified correctly
 $debug = "false"; 

# Perl Modules
# see http://www.cpan.org/modules/index.html for more details on Perl modules

 use DBI;
 $user = "username for mysql database";
 $password = "password for mysql database";
 # HTTP Perl module
 use HTTP::Request;

 # LWP Perl module
 use LWP::UserAgent;

# Constants
 $base_dir        = "absolute path to newsfeeds folder";
 $cache_dir       = "$base_dir/cached_news";
 $timeout         = 30;

# Problem characters
 # Some webservers have trouble with these - try adding to it if necessary.
 # FORMAT: '<string to replace>','<string to replace it with>'
 %problem_chars = ('\%','_CHAR', '\&','_AND_', '\|','_OR_', '!','_NOT_', '/','_SLASH_','\?');

# NO MODIFICATIONS NEEDED AFTER THIS POINT

##########

# start of program
my $dbh=DBI->connect('dbi:mysql:ENTER DATABASE NAME', $user, $password) or die "$DBI::errstr";
my $sth=$dbh->prepare('select link, id from newsfeedslinks where inuse=1');
$sth->execute or die "$DBI::errstr";

my $sth1 = $dbh->prepare("select numnews from components where module='newsfeeds'");
$sth1->execute or die "$DBI::errstr";
while (my ($num)=$sth1->fetchrow_array){
	$numnews = $num;
	}

print "Content-type: text/plain\n\n";

for ($i=0; $i<@ARGV; $i=$i+2)
{
  $arguments{$ARGV[$i]} = $ARGV[$i+1];
}

$c_field     = $arguments{c};
$o_field     = "js";
$s_field     = $arguments{s};
$k_field     = $arguments{k};
$k2_field    = $arguments{k2};
$k3_field    = $arguments{k3};
$u_field     = $arguments{u};
$n_field     = $arguments{n};
$query_field = $arguments{query};
$feed_field  = $arguments{feed};

$format = $o_field;

if    ($format eq "js")   {$content_type = "javascript";}
elsif ($format eq "cf")   {$content_type = "wddx";}
elsif ($format eq "")     {$content_type = "html"; $format = "html";}
elsif ($format=~/flash/)  {$content_type = "plain";}
elsif ($format=~/tsv/)    {$content_type = "plain";}
elsif ($format eq "rss")  {$content_type = "xml";}
elsif ($format eq "wddx") {$content_type = "xml";}
elsif ($format eq "cf")   {$content_type = "plain";}
elsif ($format eq "cdf")  {$content_type = "xml";}
else                      {$content_type = $format;}

$k_field =~s/\&/\%26/g;
$k2_field =~s/\&/\%26/g;
$k3_field =~s/\&/\%26/g;
$query_field =~s/\&/\%26/g;

$news_query .= "&c="     . $c_field if $c_field ne "";
$news_query .= "&k="     . $k_field if $k_field ne "";
$news_query .= "&k="     . $k2_field if $k2_field ne "";
$news_query .= "&k="     . $k3_field if $k3_field ne "";
$news_query .= "&s="     . $s_field if $s_field ne "";
$news_query .= "&u="     . $u_field if $u_field ne "";
$news_query .= "&n="     . $n_field if $n_field ne "";
$news_query .= "&query=" . $query_field if $query_field ne "";
$news_query .= "&feed="  . $feed_field if $feed_field ne "";

$news_query =~s/_CHAR/%/g;
$news_query =~s/ /%20/g;
$news_query =~s/^\&//;

while (my ($news_query, $id)=$sth->fetchrow_array){
	$feed_url = "http://p.moreover.com/cgi-local/page?o=js&n=". $numnews . "&".$news_query;
	$cache_file = $news_query;
	
	$cache_file =~s/\%20/__/g;
	
	foreach $badchar (keys %problem_chars) {$cache_file=~s/$badchar/$problem_chars{$badchar}/g;}
	
	$cache_file = "$cache_dir/".$cache_file.".".$format;
	$cache_file =~ s/\015//g;
	$cache_file =~ s/\r//g;
	$cache_file =~ s/\n//g;
	print "\n$cache_file\n";
	$feed = &get_url_with_timeout($feed_url,$timeout);
	if ($feed ne "")
	{
#	  print "\nWriting feed url: $feed_url\n\nto filename: $cache_file\n\n";
	  &write_file($feed,$cache_file) or die "Error writing newsfeed file to $cache_file\n\n";
#	  print "\$feed:\n\n$feed" if $debug eq "true";
	}
	else
	{
	  print "Failed to get newsfeed from $feed_url\n";
	}
	
	my $dbh=DBI->connect('dbi:mysql:miroconstruct', $user, $password) or die "$DBI::errstr";
	my $sth=$dbh->prepare("UPDATE newsfeedslinks SET filename='$cache_file' WHERE id='$id'");
	$sth->execute or die "$DBI::errstr";
	
	
}

# end of program

##########

# start of subroutines

sub write_file
  {
  my ($outputstring,$filename) = @_;
  #print "\n\nthis is a file called $filename\n\n";
  $filename =~ s/\015//g;
  $filename =~ s/\?//g;
  print "\n\nthis is a file called $filename\n\n";
  open(PAGE,">$filename");
  print PAGE $outputstring;
  close (PAGE);
#  print "\n\nthis is a file called $filename\n\n";
  }

sub get_url_with_timeout
{
  my ($url,$timeout_length) = @_;
  my $page_returned = "";
  use LWP::UserAgent;
  $ua = new LWP::UserAgent;
  $ua->timeout($timeout_length);
  my $req = new HTTP::Request GET => "$url";
  my $res = $ua->request($req);

  if ($res->is_success) { $page_returned = $res->content; } 
  return $page_returned;
}

# end of subroutines

###########


